from .api import (
    AgenyIncidentsApi,
    AgencyUsersApi,
    AgencyStationsApi,
    AgencyClassesApi
    )


class Product():

    def __init__(self, name, path, subscription):
        self.name = name
        self.path = path
        self.subscription = subscription
        self.apis = []


class AgencyIncidentsProduct(Product):

    def __init__(self, subscription):
        super().__init__('Agency Incidents', 'agencyincidents/', subscription)
        self.AgencyIncidentsApi = AgenyIncidentsApi(subscription)
        self.AgencyUsersApi = AgencyUsersApi(subscription)


class AgencyAdministrationProduct(Product):

    def __init__(self, subscription):
        super().__init__('Agency Administration', '', subscription)
        self.AgencyStationsApi = AgencyStationsApi(subscription)


class AgencyClassesProduct(Product):
    def __init__(self, subscription):
        super().__init__('Agency Classes', '', subscription)
        self.AgencyClassesApi = AgencyClassesApi(subscription)



import requests
import re
import datetime
from urllib.parse import urljoin
from .util import BASE_URL
from .excpetion import AuthException
import logging


class Token():

    def __init__(self, access_token, refresh_token, life, received_at):
        self.access_token = access_token
        self.refresh_token = refresh_token
        self.life = life
        self.recevied_at = received_at

    def age(self):
        age = datetime.datetime.utcnow() - self.recevied_at
        print(age.seconds)
        return age.seconds

    def token_too_old(self):
        if self.age() > (self.life * 0.9):
            return True
        else:
            return False


class Auth():

    def __init__(self, client_id, client_secret, username, password):
        self.client_id = client_id
        self.client_secret = client_secret
        self.username = username
        self.password = password
        self.base_url = BASE_URL
        self.code = None
        self.token = None
        self.primary_key = None

    def _get_auth_code(self):
        path = 'auth/Authorize.php'
        data = {
            "username": self.username,
            "password": self.password,
            "client_id": self.client_id,
            "response_type": "code",
            "state": "xyz"
        }
        headers = {
            'Ocp-Apim-Subscription-Key': self.primary_key
        }
        r = requests.post(urljoin(self.base_url, path), data=data, headers=headers)
        if r.ok:
            try:
                self.code = re.search(b'code=(.*)&', r.content).group(1)
            except AttributeError:
                logging.error(r.content)
                logging.error(str(r.status_code))                
                raise AuthException
        else:
            logging.error(r.content)
            logging.error(str(r.status_code))
            raise AuthException

    def _get_token(self):
        path = 'authtoken/Token.php'
        data = {
            "grant_type": "authorization_code",
            "code": self.code,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": "https://google.com"
        }
        headers = {
            'Ocp-Apim-Subscription-Key': self.primary_key
        }
        r = requests.post(urljoin(self.base_url, path), data=data, headers=headers)
        self.auth_time = datetime.datetime.utcnow()
        self.token = Token(
            r.json()['access_token'],
            r.json()['refresh_token'],
            life=r.json()['expires_in'],
            received_at=datetime.datetime.utcnow()
        )

    def get_token(self):
        self._get_auth_code()
        self._get_token()



from .subscription import BaseSubscription
from .product import (
    AgencyIncidentsProduct,
    AgencyAdministrationProduct,
    AgencyClassesProduct
    )


class EmergencyReportingClient():

    def __init__(self, client_id=None, client_secret=None, username=None, password=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.username = username
        self.password = password
        self.AgencyIncidentsProduct = AgencyIncidentsProduct(
            BaseSubscription(
                client_id=client_id,
                client_secret=client_secret,
                username=username,
                password=password
                )
            )
        self.AgencyAdministrationProduct = AgencyAdministrationProduct(
            BaseSubscription(
                client_id=client_id,
                client_secret=client_secret,
                username=username,
                password=password
                )
        )
        self.AgencyClassesProduct = AgencyClassesProduct(
            BaseSubscription(
                client_id=client_id,
                client_secret=client_secret,
                username=username,
                password=password
                )
        )

    def auth(self):
        self.AgencyIncidentsProduct.subscription.Auth.get_token()
        self.AgencyAdministrationProduct.subscription.Auth.get_token()
        self.AgencyClassesProduct.subscription.Auth.get_token()
        


from setuptools import setup, find_packages
import emergency_reporting

setup(
    name='emergency-reporting',
    packages=find_packages(),
    # license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description_content_type="text/markdown",
    long_description=open('README.md').read(),
    version=emergency_reporting.__version__,
    # use_scm_version = {"root": ".", "relative_to": __file__},
    # setup_requires=['setuptools_scm'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        # 'Programming Language :: Python :: 2',
        # 'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    install_requires=[
        "requests"
    ],

)



# Subcriptions are for products, and products contain a number of seperate API's
from urllib.parse import urljoin
import requests
from .excpetion import WebCallException
from .util import BASE_URL


class API():

    def __init__(self, name, path, subscription):
        self.base = BASE_URL
        self.name = name
        self.path = path
        self.subscription = subscription

    def url(self):
        return urljoin(self.base, self.path)

    def make_call(self, path, params, retry=1):
        print(f'Making url call retry ={retry}')
        limit = 5
        headers = {
            'Ocp-Apim-Subscription-Key': self.subscription.Auth.primary_key,
            'Authorization': self.subscription.Auth.token.access_token
        }
        url = urljoin(self.url(), path)
        response = requests.get(url, params=params, headers=headers)
        if not response.ok and retry <= limit:
            if response.status_code == 401:
                self.subscription.Auth.get_token()
            retry += 1
            return self.make_call(path, params, retry)

        return response

    def retrieve(self, path, params, json_key):
        response = self.make_call(path, params)
        if response.ok:
            return response.json()[json_key]
        else:
            raise WebCallException(response.status_code)            


class AgenyIncidentsApi(API):

    def __init__(self, subscription):
        super().__init__('Ageny Incidents', 'agencyincidents/', subscription=subscription)

    def get_incidents(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'incidents'
        json_key = 'incidents'
        return self.retrieve(path, kwargs, json_key)

    def get_exposures(self, incident_id, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = f'incidents/{incident_id}/exposures'
        json_key = 'exposures'
        return self.retrieve(path, kwargs, json_key)

    def get_all_exposures(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'incidents/exposures'
        json_key = 'exposures'
        return self.retrieve(path, kwargs, json_key)

    def get_exposure_crew_members(self, exposureID, **kwargs):
        path = f'exposures/{exposureID}/crewmembers'
        json_key = 'crewMembers'
        return self.retrieve(path, kwargs, json_key)

    def get_all_exposure_crew_members(self, **kwargs):
        path = 'exposures/crewmembers'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['crewMembers']
        else:
            raise WebCallException(response.status_code)       

    def get_crew_memeber_roles(self, exposure_user_id, **kwargs):
        path = f'crewmembers/{exposure_user_id}/roles'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['roles']
        else:
            raise WebCallException(response.status_code)

    def get_exposure_location(self, exposureID, **kwargs):
        path = f'exposures/{exposureID}/location'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['exposureLocation']
        else:
            raise WebCallException

    def get_all_exposure_locations(self, **kwargs):
        path = 'exposures/location'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['exposureLocations']
        else:
            raise WebCallException

    def get_all_exposure_naratives(self, **kwargs):
        path = 'exposures/narratives'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['narratives']
        else:
            raise WebCallException


class AgencyUsersApi(API):

    def __init__(self, subscription):
        super().__init__('Ageny Users', 'agencyusers/', subscription)

    def get_users(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'users'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['users']
        else:
            raise WebCallException(response.status_code)


class AgencyStationsApi(API):

    def __init__(self, subscription):
        super().__init__('Ageny Stations', 'agencystations/', subscription)

    def get_stations(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'stations'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['stations']
        else:
            raise WebCallException(response.status_code)


class AgencyClassesApi(API):

    def __init__(self, subscription):
        super().__init__('Ageny Classes', 'agencyclasses/', subscription)

    def get_classes(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'classes'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['classes']
        else:
            raise WebCallException(response.status_code)

    def get_all_classes_students(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'classes/students'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['students']
        else:
            raise WebCallException(response.status_code)

    def get_all_classes_instructors(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'classes/instructors'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['instructors']
        else:
            raise WebCallException(response.status_code)

    def get_all_agency_code_categories(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'classes/agencycodecategories'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['agencyCodeCategories']
        else:
            raise WebCallException(response.status_code)

    def get_all_agency_codes(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'classes/agencycodes'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['agencyCodes']
        else:
            raise WebCallException(response.status_code)

    def get_all_classes_categories(self, **kwargs):
        '''
        kwargs are parsed as the get params. Valid args are:-
        rowVersion, limit, offset, filter, orderby
        '''
        path = 'classes/categories'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['categories']
        else:
            raise WebCallException(response.status_code)

    def get_class_category(self, categoryID, **kwargs):
        path = f'classes/categories/{categoryID}'
        response = self.make_call(path, kwargs)
        if response.ok:
            return response.json()['category']
        else:
            raise WebCallException


from .auth import Auth


class BaseSubscription():

    def __init__(
            self, client_id, client_secret,
            username, password,
            primary_key=None, secondary_key=None):
        self.Auth = Auth(client_id, client_secret, username, password)
        self.Auth.primary_key = primary_key
